"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require("./");
const parser_1 = require("./parser");
/**
 * Defines a document parser.
 * @public
 */
class DocumentParser {
    constructor(tokenizer) {
        this.tokenizer = tokenizer;
    }
    /**
     * Defines how a set of nodes should be parsed
     * @public
     * @param path - a node name of a path to a node (similar to file path, this is not a xpath)
     * @param parser - a parser to be applied
     */
    on(path, parser) {
        if (this.openElement) {
            if (this.openElement.parser instanceof parser_1.Parser.Switch) {
                this.openElement.parser.case(parser_1.Parser.PassThrough.on(path, parser));
            }
            else {
                this.openElement.parser = parser_1.Parser.Switch.create()
                    .case(this.openElement.parser)
                    .case(parser_1.Parser.PassThrough.on(path, parser));
            }
        }
        else {
            this.openElement = {
                parser: parser_1.Parser.PassThrough.on(path, parser),
                ctx: undefined,
            };
        }
        return this.openElement.parser;
    }
    /**
     * Defines, as for `on()` how the whole document should be parsed
     * @param parser - parser to be applied
     */
    onRoot(parser) {
        this.openElement = {
            parser: {
                onChild(_) {
                    return parser;
                },
            },
            ctx: undefined,
        };
    }
    /**
     * Writes xml chunks to the parser
     * @remarks
     * Before writing a new chunk the current one should be exhausted (all tokens have been processed, this means {@link DocumentParser.next} has returned `undefined`)
     * @param chunk- byte sequence (utf8)
     */
    write(chunk) {
        this.tokenizer.write(chunk);
    }
    /**
     * Getting the next element
     */
    next() {
        var _a;
        let token;
        while (this.openElement && (token = this.tokenizer.nextToken())) {
            if (token instanceof _1.Token.StartTag) {
                const ctx = this.resolveTag(this.openElement, token);
                if (ctx) {
                    // selfClosing
                    return ctx;
                }
            }
            else if (token instanceof _1.Token.EndTag) {
                const ctx = this.openElement.parser.onEnd &&
                    this.openElement.parser.onEnd(this.openElement.ctx, (_a = this.openElement.parent) === null || _a === void 0 ? void 0 : _a.ctx);
                this.openElement = this.openElement.parent;
                if (ctx) {
                    return ctx;
                }
            }
            else if (token instanceof _1.Token.Text || token instanceof _1.Token.CDATA) {
                this.openElement.parser.onText && this.openElement.parser.onText(token, this.openElement.ctx);
            }
        }
        return undefined;
    }
    resolveTag(openElement, startTag) {
        const parser = openElement.parser.onChild ? openElement.parser.onChild(startTag) : openElement.parser;
        if (parser) {
            const ctx = parser.onStart && parser.onStart(startTag, openElement.ctx);
            if (!startTag.selfClosing && (ctx === false || parser.skipChildNodes)) {
                this.tokenizer.skipChildNodes();
            }
            if (startTag.selfClosing) {
                return parser.onEnd && parser.onEnd(ctx, openElement.ctx);
            }
            else {
                this.openElement = {
                    parent: openElement,
                    parser,
                    ctx: ctx || openElement.ctx,
                };
            }
        }
        else if (!startTag.selfClosing) {
            this.tokenizer.skipChildNodes(true);
        }
    }
}
exports.DocumentParser = DocumentParser;
//# sourceMappingURL=document-parser.js.map