"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bs_1 = require("./bs");
/**
 * Namespace resolver
 *
 * Representing the mapping of epxected namespaces and their URI's
 *
 * Created using an object whith namespaces as keys and URI's as values
 *
 *     const nsResolver = NsResolver.create({svg: 'http://www.w3.org/2000/svg'})
 */
class NsResolver extends Array {
    /** creates an instance of `NsResolver` */
    static create(nsResolverMap) {
        const nsResolver = new NsResolver();
        for (let nsName in nsResolverMap) {
            const uriString = nsResolverMap[nsName];
            const trgName = nsName ? bs_1.BS.create(nsName) : undefined;
            nsResolver.push({ srcName: trgName, trgName, uriString, uri: bs_1.BS.create(uriString) });
        }
        return nsResolver;
    }
    /**
     * create a new resolver inheriting this one with possibly additional namespace mappings of the current XML files
     *
     *     const nsResolver = NsResolver.create({
     *        abc: 'uri://abc',
     *        lmn: 'uri://lmn',
     *        '': 'uri://uvw',
     *      })
     *      const nsResolverChild = nsResolver.forChildTag([
     *        { name: BS.create('abd'), uri: BS.create('uri://abc') },
     *        { name: undefined, uri: BS.create('uri://lmn') },
     *        { name: BS.create('uvw'), uri: BS.create('uri://uvw') },
     *      ])
     *      nsResolverChild.resolveTagName(BS.create('abd:tagName1')) === 'abc:tagName1'
     *      nsResolverChild.resolveTagName(BS.create('tagName2')) === 'lmn:tagName2'
     *      nsResolverChild.resolveTagName(BS.create('uvw:tagName3')) === 'tagName3'
     *
     */
    forChildTag(ns) {
        if (!ns || ns.length === 0)
            return this;
        const nsResolver = this.slice();
        ns.forEach(({ name: srcName, uri }) => {
            const entry = nsResolver.find(ns => ns.uri.equals(uri));
            if (entry) {
                entry.srcName = srcName;
            }
            else {
                nsResolver.push({ srcName, trgName: srcName, uri, uriString: uri.toString() });
            }
        });
        return nsResolver;
    }
    /** resolve the namespace of a fully qualified tag name */
    resolveTagName(name) {
        const coIndex = name.indexOf(0x3a /*:*/);
        if (coIndex > -1) {
            return this.replaceNs(name, coIndex);
        }
        else {
            // maybe add ns
            const ns = this.find(ns => !ns.srcName);
            if (ns && ns.trgName) {
                const nsTarget = ns.trgName;
                const mappedName = new bs_1.BS(name.length + nsTarget.length + 1);
                mappedName.set(nsTarget);
                mappedName[nsTarget.length] = 0x3a; /*:*/
                mappedName.set(name.subarray(coIndex + 1), nsTarget.length + 1);
                return mappedName;
            }
            else {
                return name;
            }
        }
    }
    replaceNs(name, coIndex) {
        const ns = this.find(ns => { var _a; return (_a = ns.srcName) === null || _a === void 0 ? void 0 : _a.isStartOf(name, coIndex); });
        if (ns && ns.srcName !== ns.trgName) {
            if (!ns.trgName)
                return name.subarray(coIndex + 1);
            if (ns.srcName && ns.srcName.length === ns.trgName.length) {
                ns.trgName.forEach((b, i) => (name[i] = b));
                return name;
            }
            else {
                const mappedName = new bs_1.BS(name.length + ns.trgName.length - coIndex);
                mappedName.set(ns.trgName);
                mappedName.set(name.subarray(coIndex), ns.trgName.length);
                return mappedName;
            }
        }
        else {
            return name;
        }
    }
    /** resolve the namespace of a fully qualified attribute name */
    resolveAtts(atts) {
        for (let i = 0; i < atts.length; i++) {
            const { name } = atts[i];
            const coIndex = name.indexOf(0x3a /*:*/);
            if (coIndex !== -1) {
                atts[i].name = this.replaceNs(name, coIndex);
            }
        }
        return atts;
    }
    /** resolve the namespace set (attributes with 'xmlns' prefix) of a tag */
    resolveNs(tagNs) {
        return tagNs.map(({ name, uri }) => {
            const ns = name ? this.find(({ srcName }) => { var _a; return (_a = srcName) === null || _a === void 0 ? void 0 : _a.equals(name); }) : this.find(({ srcName }) => !srcName);
            return ns
                ? { name: ns.trgName, uriString: ns.uriString, uri: ns.uri }
                : { name: name, uriString: uri.toString(), uri: uri };
        });
    }
}
exports.NsResolver = NsResolver;
//# sourceMappingURL=ns-resolver.js.map